@@ -8,9 +8,11 @@ import os, pickle, sys, time, types, datetime, importlib
 from ast import literal_eval
 from base64 import decodebytes as base64_decodebytes, encodebytes as base64_encodebytes
 from io import BytesIO
+from hashlib import md5
 
 from reportlab.lib.rltempfile import get_rl_tempfile, get_rl_tempdir
-from . rl_safe_eval import rl_safe_exec, rl_safe_eval, safer_globals
+from . rl_safe_eval import rl_safe_exec, rl_safe_eval, safer_globals, rl_extended_literal_eval
+from PIL import Image
 
 class __UNSET__:
     @staticmethod
@@ -22,11 +24,6 @@ class __UNSET__:
     __repr__ = __str__
 __UNSET__ = __UNSET__()
 
-try:
-    from hashlib import md5
-except ImportError:
-    import md5
-
 try:
     import platform
     isPyPy = platform.python_implementation()=='PyPy'
@@ -58,7 +55,7 @@ def isNative(v):
 _rl_NoneType=type(None)
 strTypes = (str,bytes)
 def _digester(s):
-    return md5(s if isBytes(s) else s.encode('utf8')).hexdigest()
+    return md5(s if isBytes(s) else s.encode('utf8'),usedforsecurity=False).hexdigest()
 
 def asBytes(v,enc='utf8'):
     if isinstance(v,bytes): return v
@@ -389,17 +386,6 @@ def recursiveImport(modulename, baseDir=None, noCWD=0, debug=0):
         if debug:
             print('===== restore sys.path=%s' % repr(opath))
 
-# Image Capability Detection.  Set a flag haveImages
-# to tell us if PIL library is present.
-# define PIL_Image as either None, or an alias for the PIL.Image
-# module, as there are 2 ways to import it
-try:
-    from PIL import Image
-except ImportError:
-    try:
-        import Image
-    except ImportError:
-        Image = None
 haveImages = Image is not None
 
 class ArgvDictValue:
@@ -652,46 +638,19 @@ class ImageReader:
         else:
             try:
                 from reportlab.rl_config import imageReaderFlags
-                self.fp = open_for_read(fileName,'b')
-                if isinstance(self.fp, BytesIO): imageReaderFlags=0 #avoid messing with already internal files
-                if imageReaderFlags>0:  #interning
-                    data = self.fp.read()
-                    if imageReaderFlags&2:  #autoclose
-                        try:
-                            self.fp.close()
-                        except:
-                            pass
-                    if imageReaderFlags&4:  #cache the data
-                        if not self._cache:
-                            from rl_config import register_reset
-                            register_reset(self._cache.clear)
-                        data=self._cache.setdefault(_digester(data),data)
-                    self.fp=BytesIO(data)
-                elif imageReaderFlags==-1 and isinstance(fileName,str):
-                    #try Ralf Schmitt's re-opening technique of avoiding too many open files
-                    self.fp.close()
-                    del self.fp #will become a property in the next statement
-                    self.__class__=LazyImageReader
-                if haveImages:
-                    #detect which library we are using and open the image
-                    if not self._image:
-                        self._image = self._read_image(self.fp)
-                        self.check_pil_image_size(self._image)
-                    if getattr(self._image,'format',None)=='JPEG': self.jpeg_fh = self._jpeg_fh
-                else:
-                    from reportlab.pdfbase.pdfutils import readJPEGInfo
-                    try:
-                        self._width,self._height, c, dpi = readJPEGInfo(self.fp)
-                    except:
-                        annotateException('\nImaging Library not available, unable to import bitmaps only jpegs\nfileName=%r identity=%s'%(fileName,self.identity()))
-                    size = self._width*self._height*c
-                    if self._max_image_size is not None and size>self._max+image_size:
-                        raise MemoryError('JPEG %s color %s x %s image would use %s > %s bytes'
-                                            %(c,self._width,self._height,size,self._max_image_size))
+                if imageReaderFlags != 0:
+                    raise ValueError('imageReaderFlags values other than 0 are no longer supported; all images are interned now')
+                fp = open_for_read(fileName,'b')
+                if not isinstance(fp, BytesIO):
+                    tfp, fp = fp, BytesIO(fp.read())
+                    tfp.close()
+                    del tfp
+                self.fp = fp
+                self._image = self._read_image(self.fp)
+                self._image.fileName = fileName if isinstance(fileName,str) else repr(fileName)
+                self.check_pil_image_size(self._image)
+                if getattr(self._image,'format',None)=='JPEG':
                     self.jpeg_fh = self._jpeg_fh
-                    self._data = self.fp.read()
-                    self._dataA=None
-                    self.fp.seek(0)
             except:
                 annotateException('\nfileName=%r identity=%s'%(fileName,self.identity()))
 
@@ -783,13 +742,7 @@ class ImageReader:
             return None
 
 class LazyImageReader(ImageReader): 
-    def fp(self): 
-        return open_for_read(self.fileName, 'b') 
-    fp=property(fp) 
-
-    def _image(self):
-        return self._read_image(self.fp)
-    _image=property(_image) 
+    pass #now same as base class since we intern everything
 
 def getImageData(imageFileName):
     "Get width, height and RGB pixels from image file.  Wraps PIL"
@@ -1308,10 +1261,11 @@ class TimeStamp:
             self.tzname = 'UTC'
         else:
             t = time.time()
-            lt = tuple(time.localtime(t))
-            dhh = int(time.timezone / (3600.0))
-            dmm = (time.timezone % 3600) % 60
-            self.tzname = '' 
+            local_time = time.localtime(t)
+            lt = tuple(local_time)
+            dhh = int(local_time.tm_gmtoff / (3600.0))
+            dmm = (local_time.tm_gmtoff % 3600) % 60
+            self.tzname = local_time.tm_zone
         self.t = t
         self.lt = lt
         self.YMDhms = tuple(lt)[:6]
@@ -1331,10 +1285,10 @@ class TimeStamp:
 
 def recursiveGetAttr(obj, name, g=None):
     "Can call down into e.g. object1.object2[4].attr"
-    if not isStr(name): raise TypeError('invalid reursive acess using %r' % name)
+    if not isStr(name): raise TypeError('invalid recursive access of %s.%s' % (repr(obj),name))
     name = asNative(name)
     name = name.strip()
-    if not name: raise ValueError('empty recursive access')
+    if not name: raise ValueError('empty recursive access of %s' % repr(obj))
     dot = '.' if name and name[0] not in '[.(' else ''
     return rl_safe_eval('obj%s%s'%(dot,name), g={}, l=dict(obj=obj))
 
@@ -1373,3 +1327,29 @@ def yieldNoneSplits(L):
         except ValueError:
             yield L[i:]
             break
+
+def _rl_docdent(s):
+    return '\n'.join((_.lstrip() for _ in s.split('\n')))
+
+class KlassStore:
+    def __init__(self,lim=127):
+        self.lim = lim
+        self.store = {}
+
+    def add(self,k,v):
+        if not (isinstance(k,str) and isinstance(v,type)):
+            raise ValueError(f'{self.__class__.__name__}.add takes (str,type) arguments not ({type(k),type(v)})') 
+        store = self.store
+        store[k] = v
+        if len(store)>=self.lim:
+            for _ in list(store.keys())[:-self.lim]:
+                del store[_]
+
+    def __contains__(self,k):
+        return k in self.store
+
+    def __getitem__(self,k):
+        return self.store[k]
+
+    def get(self,k,default=None):
+        return self.store.get(k,default)
