@@ -11,10 +11,7 @@ their own width and how to write themselves into PDF files."""
 import os
 import marshal
 import time
-try:
-    from hashlib import md5
-except ImportError:
-    from md5 import md5
+from hashlib import md5
 
 from reportlab.pdfbase import pdfmetrics
 from reportlab.pdfbase._cidfontdata import allowedTypeFaces, allowedEncodings, CIDFontInfo, \
@@ -88,7 +85,7 @@ class CIDEncoding(pdfmetrics.Encoding):
                 self.parseCMAPFile(name)
 
     def _hash(self, text):
-        hasher = md5()
+        hasher = md5(usedforsecurity=False)
         hasher.update(text)
         return hasher.digest()
 
