@@ -40,9 +40,9 @@ except ImportError:
 def getFontName():
     try:
         from reportlab.pdfbase.pdfmetrics import registerFont
-        from reportlab.pdfbase.ttfonts import TTFont
+        from reportlab.pdfbase.ttfonts import freshTTFont
         fontName = 'Vera'
-        registerFont(TTFont(fontName, "Vera.ttf"))
+        registerFont(freshTTFont(fontName, "Vera.ttf"))
     except:
         fontName = 'Helvetica'
     return fontName
@@ -330,16 +330,42 @@ def eps():
     else:
         return ['eps']
 
+html = {}
+def writeIndex():
+    if not html: return
+    src = [].append
+    for fnRoot, formats in sorted(html.items()):
+        links = [].append
+        images = [].append
+        for x in sorted(formats):
+            if x in ('pdf', 'py', 'eps', 'ps'):
+                links(f'&nbsp;&nbsp;<a href="./{fnRoot}.{x}">{x}</a>')
+            else:
+                images(f'<p><strong>{x}</strong> <img src="./{fnRoot}.{x}" style="vertical-align:top"/></p>')
+        links = ''.join(links.__self__)
+        images = ''.join(images.__self__)
+        src(f'<p><strong style="font-size: larger">{fnRoot}</strong>{links}</p>{images}')
+    src = src.__self__
+    if not src: return
+    src = '\n'.join(src)
+    with open(os.path.join(outputfile('charts-out'),'index.html'),'w') as f:
+        f.write(f'<html>\n<body>\n{src}\n</body>\n</html>\n')
+
 def run_samples(L):
     outDir = outputfile('charts-out')
     global fontName
     oFontName = fontName
     def innerRun(formats):
         for k,f,kind in L:
-            d = f()
+            try:
+                d = f()
+            except:
+                continue
             if not renderPM: d.__dict__['preview'] = 0
             if not isinstance(d,Drawing): continue
-            d.save(formats=formats,outDir=outDir, fnRoot='test_graphics_charts_%s_%s' % (kind,k))
+            fnRoot = f'test_graphics_charts_{kind}_{k}'
+            d.save(formats=formats, outDir=outDir, fnRoot=fnRoot)
+            html.setdefault(fnRoot,set()).update(formats)
     innerRun(['pdf']+(['gif'] if renderPM else [])+['svg', 'py']+eps())
     fontName = 'Helvetica'
     innerRun(['ps'])
@@ -1056,14 +1082,36 @@ class ChartTestCase(unittest.TestCase):
             d.add(Hatching(spacings=(5,5),angles=(45,-45),xyLists=[5,5, 5,100, 99,105, 105,5],strokeWidth=1,strokeColor=colors.toColor('red'), strokeDashArray=None))
             return d
 
+        def sample_shading():
+            from reportlab.graphics.widgets.grids import ShadedRect, ShadedPolygon
+            from reportlab.lib.colors import blue, red, yellow, green
+
+            d = Drawing(400,200)
+            d.add(ShadedRect(),name='SR0')
+            d.SR0.fillColorEnd   = blue
+            d.SR0.fillColorStart = red
+            d.SR0.strokeColor    = yellow
+            d.add(ShadedPolygon(),name='SP0')
+            d.SP0.points  = [110,  110, 160, 180, 210, 110]
+            d.add(ShadedRect(x=220,y=10,width=50,height=180,fillColorStart=green, fillColorEnd=yellow,strokeColor=None,cylinderMode=True),name='SR1')
+            d.add(ShadedPolygon(points=[280,10,280,190,330,190,330,10],fillColorStart=green, fillColorEnd=yellow,strokeColor=None,cylinderMode=True,angle=10),name='SP1')
+            d.add(ShadedPolygon(points=[340,10,340,190,390,190,390,10],fillColorStart=green, fillColorEnd=yellow,strokeColor=None,cylinderMode=True,angle=45),name='SP2')
+            return d
+
         def extract_samples():
             S = [].extend
-            for m in ('barcharts','doughnut','linecharts','lineplots','piecharts','spider'):
+            from inspect import getmembers, isfunction, isclass
+            for m in ('barcharts','doughnut','linecharts','lineplots','piecharts','spider', 'legends',
+                        'dotbox', 'slidebox', 'textlabels'):
                 mod = {}
-                exec('from reportlab.graphics.charts import %s as mod' % m ,mod)
+                exec(f'from reportlab.graphics.charts import {m} as mod',mod)
                 mod  = mod['mod']
-                L = [(k,getattr(mod,k)) for k in dir(mod) if k.lower().startswith('sample')]
-                S([(k,v,m) for k,v in L if callable(v)])
+                def sameMod(v,modName=mod.__name__):
+                    return (isfunction(v) or isclass(v)) and v.__module__==modName
+                M = getmembers(mod,sameMod)
+                S([(k,v,m) for k,v in M if isfunction(v) and k.lower().startswith('sample')])
+                S([(f'{k}().demo',v().demo,m) for k,v in M if isclass(v)
+                            and k not in ('BarChart','BarChart3D') and hasattr(v,'demo')])
             return S.__self__
 
         run_samples([(k,v,'axes') for k,v in locals().items() if k.lower().startswith('sample')])
@@ -1108,6 +1156,97 @@ class ChartTestCase(unittest.TestCase):
             return drawing
         run_samples([(k,v,'axes') for k,v in locals().items() if k.lower().startswith('sample')])
 
+    @rlSkipIf(rlextraNeeded(),'rlextra needed')
+    def test_ddf_labels(self):
+        getFontName()
+        from reportlab.graphics.charts.piecharts import Pie, WedgeLabel
+        from reportlab.graphics.charts.doughnut import Doughnut
+        from reportlab.graphics.charts.spider import SpiderChart, SpokeLabel, StrandLabel
+        from reportlab.platypus import Paragraph, XPreformatted
+        from reportlab.lib.styles import getSampleStyleSheet, ParagraphStyle, str2alignment
+        def samplePieDDFLabels():
+            d = Drawing(width=200,height=200)
+            pie = Pie()
+            d.add(pie)
+            pie.x = 30
+            pie.y = 30
+            pie.width = 130
+            pie.height = 130
+            ss = getSampleStyleSheet()
+            pie.labels = ('<u>a<sup><span color="red">2</span></sup></u><br/>2', 'b<sup><span color="red">2</span></sup>','c','d')
+            DWL = type('DWL',(WedgeLabel,),{})
+            dwlStyle = ss['Normal'].clone('DWL',fontSize=12,autoLeading='min')
+            pie.labelClass = lambda : DWL(ddfKlass=Paragraph,ddfStyle=dwlStyle)
+            #pie.labelClass = lambda : DWL(ddfKlass=XPreformatted)
+            pie.slices.label_textAnchor = 'middle'
+            pie.slices.fontName = fontName
+            pie.simpleLabels = 0
+            return d
+
+        def sampleDoughnutDDFLabels():
+            d = Drawing(width=200,height=200)
+            dnut = Doughnut()
+            d.add(dnut)
+            dnut.x = 30
+            dnut.y = 30
+            dnut.width = 130
+            dnut.height = 130
+            ss = getSampleStyleSheet()
+            dnut.labels = ('<u>a<sup><span color="red">2</span></sup></u><br/>2', 'b<sup><span color="red">2</span></sup>','c','d')
+            DWL = type('DWL',(WedgeLabel,),{})
+            dwlStyle = ss['Normal'].clone('DWL',fontSize=12,autoLeading='min')
+            dnut.labelClass = lambda : DWL(ddfKlass=Paragraph,ddfStyle=dwlStyle)
+            #dnut.labelClass = lambda : DWL(ddfKlass=XPreformatted)
+            dnut.slices.label_textAnchor = 'middle'
+            dnut.slices.fontName = fontName
+            dnut.simpleLabels = 0
+            return d
+
+        def sampleSpiderDDFLabels():
+            d = Drawing(width=200,height=200)
+            sp = SpiderChart()
+            d.add(sp)
+            sp.x = 30
+            sp.y = 30
+            sp.width = 130
+            sp.height = 130
+            ss = getSampleStyleSheet()
+            sp.data = [[10,12,14,16,14,12], [6,8,10,12,9,15],[7,8,17,4,12,8]]
+            SKL = type('SKL',(SpokeLabel,),{})
+            sklStyle = ss['Normal'].clone('SKL',fontSize=12,autoLeading='min')
+            sp.spokeLabelClass = lambda : SKL(ddfKlass=Paragraph,ddfStyle=sklStyle)
+            sp.spokeLabels.textAnchor='middle'
+            sp.spokeLabels.fontName = fontName
+            sp.spokes.labelRadius     = 1.15
+            #sp.spokeLabelClass = lambda : SKL(ddfKlass=XPreformatted)
+            sp.labels = ['<u>U<sup><span color="red">2</span></sup></u><br/>3','V<sup><span color="red">3</span></sup>','W','X','Y','Z']
+            sp.strands.strokeWidth = 1
+            SDL = type('SDL',(StrandLabel,),{})
+            sdlStyle = ss['Normal'].clone('SDL',fontSize=12,autoLeading='min')
+            sp.strandLabelClass = lambda : SDL(ddfKlass=Paragraph,ddfStyle=sdlStyle)
+            sp.strands[0].fillColor = colors.pink
+            sp.strands[1].fillColor = colors.lightblue
+            sp.strands[2].fillColor = colors.palegreen
+            sp.strands[0].strokeColor = colors.red
+            sp.strands[1].strokeColor = colors.blue
+            sp.strands[2].strokeColor = colors.green
+            sp.strands.symbol = "FilledDiamond"
+            sp.strands[1].symbol = makeMarker("Circle")
+            sp.strands[1].symbol.strokeWidth = 0.5
+            sp.strands[1].symbol.fillColor = colors.yellow
+            sp.strands.symbolSize = 6
+            sp.strandLabels.fontName = fontName
+            sp.strandLabels[0,3]._text = 'special'
+            sp.strandLabels[0,1]._text = 'one'
+            sp.strandLabels[0,0]._text = 'zero'
+            sp.strandLabels[1,0]._text = 'Earth<sup>1</sup>'
+            sp.strandLabels[1,0].dR = -7
+            sp.strandLabels[2,2]._text = 'Mars'
+            sp.strandLabels.format = 'values'
+            sp.strandLabels.dR = -5
+            return d
+        run_samples([(k,v,'ddf') for k,v in locals().items() if k.lower().startswith('sample')])
+
     def test_legends(self):
         from reportlab.graphics.charts.legends import Legend, LineLegend, LineSwatch
 
@@ -1227,4 +1366,5 @@ def makeSuite():
 #noruntests
 if __name__ == "__main__":
     unittest.TextTestRunner().run(makeSuite())
+    writeIndex()
     printLocation()
